@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2007 Frank Denis. All rights reserved.
+ * Copyright (c) 2004-2008 Frank Denis. All rights reserved.
  * 
  * This crucial part of UCARP is derived from the OpenBSD project.
  * Original copyright follows. 
@@ -59,15 +59,14 @@ static void carp_set_state(struct carp_softc *sc, int state)
         break;
     case BACKUP:
         logfile(LOG_WARNING, _("Switching to state: BACKUP"));
-	if ((sc->sc_state != INIT) || (neutral != 1)) {
-	    (void) spawn_handler(dev_desc_fd, downscript);
-	}
-        gratuitous_arp(dev_desc_fd, 0);
+        if ((sc->sc_state != INIT) || (neutral != 1)) {
+            (void) spawn_handler(dev_desc_fd, downscript);
+        }
         break;
     case MASTER:
         logfile(LOG_WARNING, _("Switching to state: MASTER"));
         (void) spawn_handler(dev_desc_fd, upscript);
-        gratuitous_arp(dev_desc_fd, 1);
+        gratuitous_arp(dev_desc_fd);        
         break;
     default:
         logfile(LOG_ERR, _("Unknown state: [%d]"), (int) state);
@@ -243,12 +242,12 @@ static void carp_send_ad(struct carp_softc *sc)
     eh.ether_shost[4] = 0x00;
     eh.ether_shost[5] = vhid;
     
-    eh.ether_dhost[0] = 0xff;
-    eh.ether_dhost[1] = 0xff;
-    eh.ether_dhost[2] = 0xff;
-    eh.ether_dhost[3] = 0xff;
-    eh.ether_dhost[4] = 0xff;
-    eh.ether_dhost[5] = 0xff;        
+    eh.ether_dhost[0] = 0x01;
+    eh.ether_dhost[1] = 0x00;
+    eh.ether_dhost[2] = 0x5e;
+    eh.ether_dhost[3] = 0x00;
+    eh.ether_dhost[4] = 0x00;
+    eh.ether_dhost[5] = 0x12;        
     
     eh.ether_type = htons(ETHERTYPE_IP);    
     
@@ -262,7 +261,7 @@ static void carp_send_ad(struct carp_softc *sc)
     ip_ptr[offsetof(struct ip, ip_sum) + 1] = sum & 0xff;
 
     do {
-	rc = write(dev_desc_fd, pkt, eth_len);
+        rc = write(dev_desc_fd, pkt, eth_len);
     } while (rc < 0 && errno == EINTR);
     if (rc < 0) {
         logfile(LOG_WARNING, _("write() has failed: %s"), strerror(errno));
@@ -298,12 +297,12 @@ static void carp_send_ad(struct carp_softc *sc)
     ALLOCA_FREE(pkt);
     
     if (sc->sc_delayed_arp > 0)
-	sc->sc_delayed_arp--;
+        sc->sc_delayed_arp--;
     if (sc->sc_delayed_arp == 0) {
-	if (sc->sc_state == MASTER) {
-	    gratuitous_arp(dev_desc_fd, 1);
-	}
-	sc->sc_delayed_arp = -1;
+        if (sc->sc_state == MASTER) {
+            gratuitous_arp(dev_desc_fd);
+        }
+        sc->sc_delayed_arp = -1;
     }    
     if (advbase != 255 || advskew != 255) {
         sc->sc_ad_tmo.tv_sec = now.tv_sec + tv.tv_sec;
@@ -323,10 +322,14 @@ static void carp_setrun(struct carp_softc *sc, sa_family_t af)
 #ifdef DEBUG
     logfile(LOG_DEBUG, "carp_setrun()");
 #endif
+    if (gettimeofday(&now, NULL) != 0) {
+        logfile(LOG_WARNING, _("initializing now to gettimeofday() failed: %s"),
+                strerror(errno));
+    }
     switch (sc->sc_state) {
     case INIT:
-	carp_set_state(sc, BACKUP);
-	carp_setrun(sc, 0);	    
+        carp_set_state(sc, BACKUP);
+        carp_setrun(sc, 0);         
         break;
     case BACKUP:
         sc->sc_ad_tmo.tv_sec = 0;
@@ -379,9 +382,8 @@ static void carp_master_down(struct carp_softc *sc)
     case BACKUP:
         carp_set_state(sc, MASTER);
         carp_send_ad(sc);
-        gratuitous_arp(dev_desc_fd, 0);
-	/* Schedule a delayed ARP request to deal w/ some L3 switches */
-	sc->sc_delayed_arp = 2;
+        /* Schedule a delayed ARP request to deal w/ some L3 switches */
+        sc->sc_delayed_arp = 2;
 #ifdef INET6
         carp_send_na(sc);
 #endif /* INET6 */
@@ -429,7 +431,7 @@ static void packethandler(unsigned char *dummy,
     caplen = header->caplen - sizeof etherhead;
     memcpy(&iphead, sp, sizeof iphead);    
     if (iphead.ip_src.s_addr == srcip.s_addr) {
-	return;
+        return;
     }
     ip_len = iphead.ip_hl << 2;
     source = ntohl(iphead.ip_src.s_addr);
@@ -560,16 +562,38 @@ static void packethandler(unsigned char *dummy,
         case MASTER:
             /*
              * If we receive an advertisement from a master who's going to
-             * be more frequent than us, go into BACKUP state.
+             * be more frequent than us, or from a master who's advertising
+             * with the same frequency as us but with a lower IP address, 
+             * go into BACKUP state.
              */
-            if (timercmp(&ch_tv, &sc_tv, <=)) {
-                sc.sc_ad_tmo.tv_sec = 0;
+            if (timercmp(&sc_tv, &ch_tv, >) ||
+                (timercmp(&sc_tv, &ch_tv, ==) &&
+                 iphead.ip_src.s_addr < srcip.s_addr)) {
+		/* To be really sure the new master knows that is
+		 * has to reassert control of the VIP */
+		carp_send_ad(&sc);
+		
                 carp_set_state(&sc, BACKUP);
                 carp_setrun(&sc, 0);
                 logfile(LOG_WARNING, _("Preferred master advertised: "
                                        "going back to BACKUP state"));
             }
-            break;
+
+            /*
+             * If we receive an advertisement from a master who's advertising
+             * less frequently than us, or with the same frequency as us but
+             * with a higher IP address, reassert our dominance by issuing
+             * another gratuitous arp.
+             */
+            if (timercmp(&sc_tv, &ch_tv, <) ||
+                (timercmp(&sc_tv, &ch_tv, ==) &&
+		 iphead.ip_src.s_addr > srcip.s_addr)) {
+		gratuitous_arp(dev_desc_fd);
+		sc.sc_delayed_arp = 2; /* and yet another in 2 ticks */
+		logfile(LOG_WARNING, _("Non-preferred master advertising: "
+				       "reasserting control of VIP with another gratuitous arp"));
+            }
+	    break;
         case BACKUP:
             /*
              * If we're pre-empting masters who advertise slower than us,
@@ -616,11 +640,23 @@ RETSIGTYPE sighandler_exit(const int sig)
             downscript);
 #endif
     if (sc.sc_state != BACKUP) {
-	(void) spawn_handler(dev_desc_fd, downscript);
+        (void) spawn_handler(dev_desc_fd, downscript);
     }
     _exit(EXIT_SUCCESS);
 }
 
+RETSIGTYPE sighandler_usr(const int sig)
+{
+    switch (sig) {
+    case SIGUSR1:
+	received_signal=1;
+	break;
+    case SIGUSR2:
+	received_signal=2;
+	break;
+    }
+}
+
 char *build_bpf_rule(void)
 {
     static char rule[256];
@@ -642,11 +678,13 @@ char *build_bpf_rule(void)
 int docarp(void)
 {
     char errbuf[PCAP_ERRBUF_SIZE];
+    struct ip_mreq req_add;   
     struct bpf_program bpfp;
     struct pollfd pfds[1];
     struct ifreq iface;
     int fd;
     int nfds;
+    int iface_running = 1;
 
     sc.sc_vhid = vhid;
     sc.sc_advbase = advbase;
@@ -705,50 +743,96 @@ int docarp(void)
         (void) signal(SIGQUIT, sighandler_exit);
         (void) signal(SIGTERM, sighandler_exit);
     }
+    (void) signal(SIGUSR1, sighandler_usr);
+    (void) signal(SIGUSR2, sighandler_usr);
 
     if (gettimeofday(&now, NULL) != 0) {
-	logfile(LOG_WARNING, _("initializing now to gettimeofday() failed: %s"),
-		strerror(errno));
+        logfile(LOG_WARNING, _("initializing now to gettimeofday() failed: %s"),
+                strerror(errno));
     }
     carp_setrun(&sc, 0); 
     
     if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) == -1) {
         logfile(LOG_ERR, _("Error opening socket for interface [%s]: %s"),
-		interface == NULL ? "-" : interface, strerror(errno));
+                interface == NULL ? "-" : interface, strerror(errno));
         return -1;
     }
+    memset(&req_add, 0, sizeof req_add);
+    req_add.imr_multiaddr.s_addr = inet_addr("224.0.0.18");
+    req_add.imr_interface.s_addr = srcip.s_addr;
+    if (setsockopt(fd, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+		   &req_add, sizeof req_add) < 0) {
+	logfile(LOG_ERR, "Can't do IP_ADD_MEMBERSHIP errno=%s (%d)",
+		strerror(errno), errno);
+	close(fd);
+	return -1;
+    }
 #ifdef SIOCGIFFLAGS    
     if (strlen(interface) >= sizeof iface.ifr_name) {
-	logfile(LOG_ERR, _("Interface name too long"));
-	return -1;
+        logfile(LOG_ERR, _("Interface name too long"));
+        return -1;
     }
     strncpy(iface.ifr_name, interface, sizeof iface.ifr_name);
 #endif    
     for (;;) {
 #ifdef SIOCGIFFLAGS
         if (ioctl(fd, SIOCGIFFLAGS, &iface) != 0) {
-	    break;
-	}
-	if ((iface.ifr_flags & IFF_RUNNING) == 0) {
-            carp_set_state(&sc, BACKUP);
-	    sc.sc_ad_tmo.tv_sec = 0;
-	    sc.sc_ad_tmo.tv_usec = 0;
-	    sc.sc_md_tmo.tv_sec = 0;
-	    sc.sc_md6_tmo.tv_usec = 0;
-	    sleep(SECONDS_TO_WAIT_AFTER_INTERFACE_IS_DOWN);
-	    continue;
-	}
-#endif
-        nfds = poll(pfds, (nfds_t) 1, sc.sc_advbase * 1000);
-        if (nfds == -1) {
-	    if (errno == EINTR) {
+            break;
+        }
+        if ((iface.ifr_flags & IFF_RUNNING) == 0) {
+	    if (ignoreifstate == 0) {
+		carp_set_state(&sc, BACKUP);
+		sc.sc_ad_tmo.tv_sec = 0;
+		sc.sc_ad_tmo.tv_usec = 0;
+		sc.sc_md_tmo.tv_sec = 0;
+		sc.sc_md6_tmo.tv_usec = 0;
+        	if (iface_running) {
+		    iface_running = 0;
+		}
+		sleep(SECONDS_TO_WAIT_AFTER_INTERFACE_IS_DOWN);
 		continue;
 	    }
-	    logfile(LOG_ERR, _("exiting: poll() error: %s"), strerror(errno));
-	}
-	if ((pfds[0].revents & (POLLERR | POLLHUP)) != 0) {
+        } else {
+            if (!iface_running) {
+                iface_running = 1;
+                carp_setrun(&sc, 0);
+            }
+        }
+#endif
+        if (received_signal != 0) {
+            int flag = received_signal;
+	    
+            received_signal = 0;
+            switch (flag) {
+	    case 1:
+		logfile(LOG_INFO, "%s on %s id %d", 
+			(sc.sc_state == BACKUP ? "BACKUP" : "MASTER"),
+			interface, sc.sc_vhid);
+		break;
+	    case 2:
+#ifdef DEBUG
+		logfile(LOG_DEBUG, "Caught signal (USR2) considering going down");
+#endif
+		if (sc.sc_state != BACKUP) {
+		    carp_set_state(&sc, BACKUP);
+		    sleep(3); /* hold up a sec... */
+		    carp_setrun(&sc, 0); /* now listen for 3 heartbeats, as usual */
+		    continue;
+		}
+		break;
+            }
+        }
+	
+        nfds = poll(pfds, (nfds_t) 1, sc.sc_advbase * 1000);
+        if (nfds == -1) {
+            if (errno == EINTR) {
+                continue;
+            }
+            logfile(LOG_ERR, _("exiting: poll() error: %s"), strerror(errno));
+        }
+        if ((pfds[0].revents & (POLLERR | POLLHUP)) != 0) {
             logfile(LOG_ERR, _("exiting: pfds[0].revents = %d"),
-		    pfds[0].revents);
+                    pfds[0].revents);
             break;
         }
         if (gettimeofday(&now, NULL) != 0) {
