@@ -11,9 +11,6 @@ void logfile(const int crit, const char *format, ...)
     va_list va;
     char line[MAX_SYSLOG_LINE];
     
-    if (no_syslog != 0) {
-        return;
-    }
     va_start(va, format);
     vsnprintf(line, sizeof line, format, va);
     switch (crit) {
@@ -35,22 +32,23 @@ void logfile(const int crit, const char *format, ...)
     default:
         urgency = "";
     }
+    if (no_syslog == 0) {
 #ifdef SAVE_DESCRIPTORS
-    openlog("ucarp", LOG_PID, syslog_facility);
+        openlog("ucarp", LOG_PID, syslog_facility);
 #endif
-    syslog(crit, "%s%s", urgency, line);
+        syslog(crit, "%s%s", urgency, line);
 #ifdef SAVE_DESCRIPTORS
-    closelog();
+        closelog();
 #endif
-    
+    }    
     if (daemonize == 0) {
         switch (crit) {
         case LOG_WARNING:
         case LOG_ERR:
-            printf("%s%s\n", urgency, line);
+            fprintf(stderr, "%s%s\n", urgency, line);
             break;
         default:
-            fprintf(stderr, "%s%s\n", urgency, line);
+            printf("%s%s\n", urgency, line);
         }
     }    
     va_end(va);
