@@ -27,6 +27,7 @@ static void usage(void)
 	    "--srcip=<ip> (-s <ip>): source (real) IP address of that host\n"
 	    "--vhid=<id> (-v <id>): virtual IP identifier (1-255)\n"
 	    "--pass=<pass> (-p <pass>): password\n"
+	    "--passfile=<file> (-o <file>): read password from file\n"
 	    "--preempt (-P): becomes a master as soon as possible\n"
 	    "--neutral (-n): don't run downscript at start if backup\n"
 	    "--addr=<ip> (-a <ip>): virtual shared IP address\n"
@@ -38,7 +39,9 @@ static void usage(void)
 	    "--deadratio=<ratio> (-r <ratio>): ratio to consider a host as dead\n"
 	    "--shutdown (-z): call shutdown script at exit\n"
 	    "--daemonize (-B): run in background\n"
+	    "--ignoreifstate (-S): ignore interface state (down, no carrier)\n"
 	    "--facility=<facility> (-f): set syslog facility (default=daemon)\n"
+	    "--xparam=<value> (-x): extra parameter to send to up/down scripts\n"	    
 	    "\n"
 	    "Sample usage:\n"
 	    "\n"
@@ -124,6 +127,32 @@ int main(int argc, char *argv[])
             }
             break;            
         }
+	case 'o': {
+	    char buf[512U];
+	    char *p;
+	    FILE *pw;
+	    if ((pw = fopen(optarg, "r")) == NULL) {
+		logfile(LOG_ERR,
+			_("unable to open passfile %s for reading: %s"),
+			optarg, strerror(errno));
+		return 1;
+	    }
+	    if (fgets(buf, sizeof buf, pw) == NULL) {
+		logfile(LOG_ERR, _("error reading passfile %s: %s"), optarg,
+			ferror(pw) ?
+			strerror(errno) : _("unexpected end of file"));
+		return 1;
+	    }
+	    fclose(pw);
+	    p = strchr(buf, '\n');
+	    if (p != NULL) {
+		*p = 0;
+	    }
+            if ((pass = strdup(buf)) == NULL) {
+                die_mem();
+            }
+	    break;
+	}
         case 'P': {
             preempt = 1;
             break;
@@ -133,11 +162,13 @@ int main(int argc, char *argv[])
 	    break;
 	}
         case 'a': {
+	    free(vaddr_arg);
             if (inet_pton(AF_INET, optarg, &vaddr) == 0) {
                 logfile(LOG_ERR, _("Invalid address: [%s]"), optarg);
                 return 1;
-            }            
-            break;            
+            }
+	    vaddr_arg = strdup(optarg);
+            break;
         }
         case 'b': {
             advbase = (unsigned char) strtoul(optarg, NULL, 0);
@@ -173,9 +204,13 @@ int main(int argc, char *argv[])
             daemonize = 1;
             break;
         }
+	case 'S': {
+	    ignoreifstate = 1;
+	    break;
+	}
         case 'f': {
             int n = 0;
-
+	    
             if (strcasecmp(optarg, "none") == 0) {
                 no_syslog = 1;
                 break;
@@ -190,7 +225,14 @@ int main(int argc, char *argv[])
                 logfile(LOG_ERR, _("Unknown syslog facility: [%s]"), optarg);
             }
             break;
-        }            
+        }
+	case 'x': {
+	    free(xparam);
+	    if ((xparam = strdup(optarg)) == NULL) {
+                die_mem();
+            }
+            break;
+        }
         default: {
             usage();
         }
