@@ -114,6 +114,8 @@ extern int errno;
 # define __packed__
 #endif
 
+#define _COMA_ ,
+
 #define ETHERNET_MTU 1500
 #ifndef ETHER_ADDR_LEN
 # define ETHER_ADDR_LEN 6
@@ -121,10 +123,6 @@ extern int errno;
 #ifndef IPPROTO_CARP
 # define IPPROTO_CARP 112
 #endif
-#ifndef INADDR_CARP_GROUP
-# define __OBSD_IPADDR(x)     ((u_int32_t) htonl((u_int32_t)(x)))
-# define INADDR_CARP_GROUP    __OBSD_IPADDR(0xe0000012)    /* 224.0.0.18 */
-#endif
 #ifndef timercmp
 # define timercmp(tvp, uvp, cmp) \
         (((tvp)->tv_sec == (uvp)->tv_sec) ? \
@@ -195,6 +193,7 @@ struct carp_softc {
     int sc_advbase;             /* seconds */
     int sc_init_counter;
     u_int64_t sc_counter;
+    int sc_delayed_arp;
     
     /* authentication */
 #define CARP_HMAC_PAD   64
@@ -210,6 +209,7 @@ struct carp_softc {
 #define CARP_AUTHLEN 7
 #define DEFAULT_ADVBASE 1U
 #define DEFAULT_DEAD_RATIO 3U
+#define SECONDS_TO_WAIT_AFTER_INTERFACE_IS_DOWN 10U
 
 #define DEFAULT_FACILITY LOG_DAEMON
 
